# $Id: ssUpdate.tcl,v 1.17 2000/06/06 13:40:09 gdmr Exp $
#
# Copyright 2000 Division of Informatics, University of Edinburgh
# 
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of the University of Edinburgh not be used
# in advertising or publicity pertaining to distribution of the software
# without specific, written prior permission.  The University of Edinburgh
# makes no representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied warranty.
#
# Expect-based procedures for interacting with a 3Com SS II switch

# Which user should we interact with the switch as?
set updateUsername "admin"

# This needs to be global so that the expect calls in
# the procs work properly
set spawn_id 0

# Get back to the top-level menu
proc ssTopLevel {} {
   global spawn_id
   send "\r"
   expect {
      "Select menu option (*):" {
         send "q\r"
         exp_continue
      }
      "Select menu option:" {}
      timeout {
         send_user "Timeout sending to switch, trying again...\n"
         send "\r"
         exp_continue
      }
   }
}

# Get password, open a telnet session.  Return 1 if successful,
# 0 for failure.
proc ssConnect {target password} {
   global updateUsername
   global spawn_id
   send_user "Connecting to switch at $target as \"$updateUsername\"...\n"
   set wrongPass 3
   log_user 0
   spawn telnet $target
   set timeout 2
   expect {
      "Incorrect password." {
         send_user "Wrong password.\n"
         set wrongPass [expr {$wrongPass - 1}]
         if {$wrongPass} exp_continue
      }
      "Login: " {
         send "$updateUsername\r"
         exp_continue
      }
      "Password: " {
         if {[string length $password] > 0} {
            send "$password\r"
            set wrongPass 1
         } else {
            send_user "Password required for \"$updateUsername\" at $target:"
            set to $timeout
            set timeout 30
            stty -echo
            expect_user -re "(.*)\n"
            stty echo
            set timeout $to
            send_user "\n"
            send "$expect_out(1,string)\r"
         }
         exp_continue
      }
      "Select menu *: " {
         send_user "Connected.\n"
         return 1
      }
      timeout {
         send "\r"
         exp_continue
      }
   }
   return 0
}

proc ssDisconnect {} {
   global spawn_id
   send_user "Disconnecting...\n"
   ssTopLevel
   send "logout\r"
   expect {
      "Connection closed by foreign host." {}
      "Select menu *:" {
         # Hmmm.  Must have been speaking to unit 2, 3, 4
         send "logout\r"
         exp_continue
      }
   }
}

# Create a VLAN
set errorRecoveryTag 999
proc ssCreateVlan {name tag} {
   global spawn_id
   global errorRecoveryTag
   send_user "ssCreateVlan: name $name, tag $tag\n"
   ssTopLevel
   set e 1
   send "bridge vlan create\r"
   expect {
      "Enter VLAN ID*:" {
         send "$tag\r"
         exp_continue
      }
      "Enter Local ID*:" {
         send "\r"
         exp_continue
      }
      "Enter VLAN Name*:" {
         send "[getVlanFullName $name]\r"
         exp_continue
      }
      "Select menu *:" {
         send_user "ssCreateVlan: created: tag $tag, name $name\n"
         }
      "not a valid selection." {
         set tag [incr errorRecoveryTag]
         send_user "ssCreateVlan failed: bad tag, using $tag\n"
         set e 0
         exp_continue
      }
      "in use by another VLAN" {
         set tag [incr errorRecoveryTag]
         send_user "ssCreateVlan failed: tag already in use, using $tag\n"
         set e 0
         exp_continue
      }
   }
   set e
}

# Port calculations
proc ssPort {pu} {
   expr $pu % 100
}

# Switch unit if necessary.  Assume we're connected to 1.
set ssCurrentUnit 1
proc ssSetUnit {pu} {
   global ssCurrentUnit
   set u [expr {$pu / 100}]
   if {$u == $ssCurrentUnit} {return 1}
   # If we're not speaking to unit 1 at the moment then
   # we have to revert to it first.  Assume that we've been
   # left at the top-level menu on whatever unit we're
   # currently speaking to.
   if {$ssCurrentUnit != 1} {
      ssTopLevel
      send "logout\r"
      set ssCurrentUnit 1
   }
   ssTopLevel
   # OK, we're talking to unit 1 now, so if that's not
   # the one we really want then we have to select it now
   set e 1
   if {$u != 1} {
      send "system unit $u\r"
      expect {
         "* is invalid" {
            # Eh?  Revert to 1, return error
            send_user "ssSetUnit: no unit $u on switch stack??\n"
            set u 1
            set e 0
            exp_continue
         }
         "Select unit:" {
            send "$u\r"
            exp_continue
         }
         "Select menu *:" {}
      }
      set ssCurrentUnit $u
   }
   set e
}

# Set the VLAN for a port
proc ssVlan {port tag type} {
   global spawn_id
   send_user "ssVlan: port $port, tag $tag, type $type\n"
   if {![ssSetUnit $port]} {return 0}
   set p [ssPort $port]
   ssTopLevel
   set e 1
   send "bridge vlan addPort\r"
   expect {
      "Select VLAN ID *:" {
         send "$tag\r"
         exp_continue
      }
      "Select Ethernet port *:" {
         send "$p\r"
         exp_continue
      }
      "Enter tag type *:" {
         send "$type\r"
         exp_continue
      }
      "Vlan * does not exist" {
         send_user "ssVlan failed: no tag $tag, using \"1\"\n"
         set tag 1
         set e 0
         exp_continue
      }
      "* is not a valid selection" {
         send_user "ssVlan failed: bad tag $tag, using \"1\"\n"
         set tag 1
         set e 0
         exp_continue
      }
      "Error accessing the MIB." {
         send_user "ssVlan failed: error accessing the MIB:\n"
         send_user "               tag $tag, port $port, type $type\n"
         set e 0
      }
      "Select menu *:" {}
   }
   set e
}

# Set the default VLAN for a port
proc ssDefaultVlan {port tag} {
   ssVlan $port $tag "none"
}

# Add a tagged VLAN to a port
proc ssAddTaggedVlan {port tag} {
   ssVlan $port $tag "802.1Q"
}

# Remove a VLAN from a port
proc ssRemoveVlan {port tag} {
   global spawn_id
   send_user "ssRemoveVlan: port $port, tag $tag\n"
   if {![ssSetUnit $port]} {return 0}
   set p [ssPort $port]
   ssTopLevel
   set e 1
   send "bridge vlan removePort\r"
   expect {
      "Select VLAN ID*:" {
         send "$tag\r"
         exp_continue
      }
      "Select Ethernet port*:" {
         send "$p\r"
         exp_continue
      }
      "Vlan * does not exist" {
         send_user "ssRemoveVlan failed: no tag $tag\n"
         set e 0
         exp_continue
      }
      "Port * is not a member" {
         send_user "ssRemoveVlan failed: port $port not in vlan $tag\n"
         set e 0
         exp_continue
      }
      "Select menu *:" {}
   }
   set e
}

# Delete an entire VLAN
proc ssDeleteVlan {name tag} {
   global spawn_id
   send_user "ssDeleteVlan: $tag\n"
   # Do this speaking to unit 1
   if {![ssSetUnit 100]} {return 0}
   ssTopLevel
   set e 1
   send "bridge vlan delete\r"
   expect {
      "Select VLAN ID *:" {
         send "$tag\r"
         exp_continue
      }
      "* does not exist" {
         send_user "ssDeleteVlan: no tag $tag for \"$name\"?\n"
         set e 0
         exp_continue
      }
      "* contains ports. These must be removed first" {
         send_user "ssDeleteVlan: still ports for \"$name\", tag $tag\n"
         exp_continue
      }
      "Select menu *:" {}
   }
   set e
}

# Set a port's autonegotiation
proc ssSetAuto {port {type enable}} {
   global spawn_id
   if {![ssSetUnit $port]} {return 0}
   send_user "ssSetAuto: port $port, type $type\n"
   ssTopLevel
   send "ethernet autoNegotiation [ssPort $port] $type\r"
   expect "Select menu *:"
   return 1
}

# Set a port's (explicit) speed
set speedString(10HD) 10half
set speedString(10FD) 10full
set speedString(100HD) 100half
set speedString(100FD) 100full

proc ssSetSpeed {port speed} {
   global speedString
   global spawn_id
   send_user "ssSetSpeed: port $port, speed $speed\n"
   if {![ssSetUnit $port]} {return 0}
   set p [ssPort $port]
   ssTopLevel
   ssSetAuto $port "disable"
   send "ethernet portMode [ssPort $port] $speedString($speed)\r"
   set e 1
   expect {
      "Select menu *:" {}
      "* not a valid selection" {
         send_user "[textPort $port] doesn't support speed $speed\n"
         send "\r"
         set e 0
      }
   }
   set e
}
